﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.Windows;
using System.Windows.Controls;
using MVVMexample.Utils;
using MVVMexample.Views.UserControls;
using System.ComponentModel;
using System.Linq.Expressions;
using MVVMexample.ViewModels.Data;

namespace MVVMexample.ViewModels
{
    public class MainWindowVM : INotifyPropertyChanged
    {
        public List<Datasource1> Data { get; private set; }

        public void makeData()
        {
            if (Data == null) Data = new List<Datasource1>();
            for (int i = 0; i < 10; i++)
            {
                Data.Add(new Datasource1(i, randomString(3), randomString(4), randomString(5)));
            }
        }

        private string randomString(int size)
        {
            StringBuilder builder = new StringBuilder();
            Random random = new Random();
            char ch ;
            for (int i = 0; i < size; i++)
            {
                ch = Convert.ToChar(Convert.ToInt32(Math.Floor(26 * random.NextDouble() + 65)));
                builder.Append(ch);
            }
            return builder.ToString();
        }

        private RelayCommand _genReport;
        public ICommand Gen_Report {
            get {
                if (_genReport == null) {
                    _genReport = new RelayCommand(param => this.generateReport());
                }
                return _genReport;
            }
        }

        private RelayCommand _blankSignal;
        public ICommand Blank_Signal {
            get
            {
                if (_blankSignal == null)
                {
                    _blankSignal = new RelayCommand(param => this.generateBase());
                }
                return _blankSignal;
            }
   
        }

        private void generateReport() {
            this.MainContentControl = new ReportViewerControl() { DataContext = this };
        }

        private void generateBase()
        {
            this.MainContentControl = new BaseControl() { DataContext = this };
        }

        private UserControl mainContentControl;
        public UserControl MainContentControl {
            get { return this.mainContentControl; }
            set {
                if (this.mainContentControl != null) {
                    this.CheckAndDispose(this.mainContentControl.DataContext);
                    this.mainContentControl.DataContext = null;
                }
                this.mainContentControl = value;
                this.OnPropertyChanged(() => this.MainContentControl);
            }
        }

        private string _reportDefinition = "MVVMexample.Reports.Report1.igr, MVVMexample";
        public String ReportDefinition
        {
            get
            {
                return this._reportDefinition;
            }
        }

        private string _reportSource = "Datasource1";
        public string ReportSource
        {
            get
            {
                return _reportSource;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
        private void OnPropertyChanged(string propertyName) {
            if (this.PropertyChanged != null) {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private void OnPropertyChanged<TResult>(Expression<Func<TResult>> propertyExpression) {
            this.OnPropertyChanged(((MemberExpression) propertyExpression.Body).Member.Name);
        }

        private void CheckAndDispose(object component) {
            if (component is IDisposable) {
                (component as IDisposable).Dispose();
            }
        }
    }
}
